#version 300 es

uniform highp mat4 _ViewProjectionMatrix;
uniform highp mat4 modelMatrix;
uniform highp mat4 _ViewMatrix;
uniform highp mat4 _TPMatrix;
uniform highp mat4 boneMatrices[8];
uniform highp vec4 diffuseMap_ST;
uniform highp vec3 _LightDirection;
uniform highp vec3 _LightPosition;
uniform highp vec3 _CameraPosition;
uniform highp int _LightType;

layout (location = 0) in highp vec4 aPosition;
layout (location = 1) in highp vec2 aTexCoord;
layout (location = 2) in highp vec3 aNormal;
layout (location = 3) in highp vec4 aTangent;
layout (location = 4) in highp vec4 aBoneIndex;
layout (location = 5) in highp vec4 aBoneWeights;

out highp vec2 vTexCoord;
out highp vec3 vViewVec;
out highp vec3 vLightVec;
#ifdef ENVIRONMENT_MAP
out vec3 vEnvTexCoord;
#endif

// Shadow stuff
out highp vec4 vPosition;
out highp vec4 vProjCoord;

void main()
{
    highp vec4 position = aPosition;
#ifdef SKIN
    highp ivec4 boneIndex = ivec4( aBoneIndex );

    position  = boneMatrices[ boneIndex.x ] * aPosition * aBoneWeights.x;
    position += boneMatrices[ boneIndex.y ] * aPosition * aBoneWeights.y;
    position += boneMatrices[ boneIndex.z ] * aPosition * aBoneWeights.z;
    position += boneMatrices[ boneIndex.w ] * aPosition * aBoneWeights.w;
#endif

    gl_Position = _ViewProjectionMatrix * modelMatrix * position;
    
    highp mat4 modelViewMatrix = _ViewMatrix * modelMatrix;
    highp vec4 positionVS = modelViewMatrix * position;
    
    highp mat3 normalMatrix = mat3( modelViewMatrix );
    
    highp vec3 t = normalize( normalMatrix * aTangent.xyz );
    highp vec3 b = normalize( normalMatrix * (cross( aNormal, aTangent.xyz ) * aTangent.w) );
    highp vec3 n = normalize( normalMatrix * aNormal );
    highp mat3 tbn = transpose( mat3( t, b, n ) );

    // Directional.
    if (_LightType == 1)
    {
        vLightVec = (_ViewMatrix * vec4( -_LightDirection, 0.0 )).xyz;
    }
    else
    {
        highp vec3 dirToLight = normalize( _LightPosition - _CameraPosition );
        vLightVec = (_ViewMatrix * vec4( dirToLight, 0.0 )).xyz;
    }

    vLightVec = tbn * vLightVec;
    vViewVec = -positionVS.xyz;
    vViewVec = tbn * vViewVec;

    vTexCoord = aTexCoord * diffuseMap_ST.xy + diffuseMap_ST.zw;
    
#ifdef ENVIRONMENT_MAP
    vec4 worldPos    = modelMatrix * position;
    vec4 worldNormal = normalize( modelMatrix * vec4( aNormal, 0.0 ) );
    vec4 worldEye    = normalize( worldPos - vec4( _CameraPosition, 1.0 ) );
    
    vEnvTexCoord = reflect( worldEye.xyz, worldNormal.xyz );
#endif

    // Shadow stuff
    vPosition  = modelMatrix * position;
    vProjCoord = _TPMatrix * position;
}
