cbuffer buffer : register(b0)
{
    column_major matrix _ViewProjectionMatrix;
    column_major matrix modelMatrix;
    float4 textureMap_ST;
    float4 tint;
    float3 morphWeights;
}

struct VS_INPUT
{
    float3 Pos : POSITION;
    float2 Tex : TEXCOORD0;
    float3 Normal : NORMAL;
    float4 Tangent : TANGENT;
    float4 Color : COLOR;
    float3 Pos1 : MORPH_POS1;
    float3 Pos2 : MORPH_POS2;
};

struct PS_INPUT
{
    float4 Pos : SV_POSITION;
    float2 Tex : TEXCOORD0;
};

PS_INPUT VS( VS_INPUT input )
{
    PS_INPUT output = (PS_INPUT)0;

    float weightSum = morphWeights.x + morphWeights.y;
    float4 position = float4( input.Pos, 1.0f ) * (1.0f - weightSum);
    position += (float4( input.Pos1, 1.0f ) * morphWeights.x);
    position += (float4( input.Pos2, 1.0f ) * morphWeights.y);

    output.Pos = mul( _ViewProjectionMatrix, mul( modelMatrix, position ) );
    output.Tex = input.Tex * textureMap_ST.xy + textureMap_ST.zw;
    return output;
}

Texture2D textureMap : register(t0);
SamplerState SampleType : register(s0);

float4 PS( PS_INPUT input ) : SV_Target
{
    return tint * textureMap.Sample( SampleType, input.Tex );
    //return tint;
}