cbuffer buffer : register(b0)
{
    column_major matrix _ViewProjectionMatrix;
    column_major matrix modelMatrix;
#ifdef SKIN
    column_major matrix boneMatrices[8];
#endif
    float4 textureMap_ST;
    float4 tint;
}

struct VS_INPUT
{
    float3 Pos : POSITION;
    float2 Tex : TEXCOORD0;
    float3 Normal : NORMAL;
    float4 Tangent : TANGENT;
    float4 Color : COLOR;
#ifdef SKIN
    float4 BoneInd : BONE_INDEX;
    float4 Weights : WEIGHT_INDEX;
#endif
};

struct PS_INPUT
{
    float4 Pos : SV_POSITION;
    float2 Tex : TEXCOORD0;
};

PS_INPUT VS( VS_INPUT input )
{
    PS_INPUT output = (PS_INPUT)0;
    float4 position = float4( input.Pos, 1 );
#ifdef SKIN
    int4 boneIndex = int4( input.BoneInd );
    
    position  = mul( boneMatrices[ boneIndex.x ], float4( input.Pos, 1.0f ) * input.Weights.x );
    position += mul( boneMatrices[ boneIndex.y ], float4( input.Pos, 1.0f ) * input.Weights.y );
    position += mul( boneMatrices[ boneIndex.z ], float4( input.Pos, 1.0f ) * input.Weights.z );
    position += mul( boneMatrices[ boneIndex.w ], float4( input.Pos, 1.0f ) * input.Weights.w );
#endif

    output.Pos = mul( _ViewProjectionMatrix, mul( modelMatrix, position ) );
    output.Tex = input.Tex * textureMap_ST.xy + textureMap_ST.zw;
    return output;
}

Texture2D textureMap : register(t0);
SamplerState SampleType : register(s0);

float4 PS( PS_INPUT input ) : SV_Target
{
    return tint * textureMap.Sample( SampleType, input.Tex );
}