#ifndef AUDIO_H
#define AUDIO_H

/// Handles audio.
class Audio
{
  public:
    /// Destructor.
    virtual ~Audio() {}

    /**
       \brief Adds a sound. It can be played back using Play().

       \param soundName Sound's name.
       \param soundFile Sound's file name.  Format can be .wav or .ogg (vorbis).
       \see Play()
     */
    virtual void AddSound( const char* soundName, const char* soundFile ) = 0;

    /**
       \brief Plays a sound. It must have been added before using addSound().

       \param soundName Sound name.
     */
    virtual void Play( const char* soundName ) = 0;
    
    /**
       \brief Mute status.  Default: unmuted.

       \return True, if the audio is muted, false otherwise.
     */
    virtual bool IsMuted() const = 0;
  
    /**
     \brief Sets Doppler factor.
     
     \param factor Factor. Defaults to 1.
     */
    virtual void SetDopplerFactor( float factor ) = 0;

    /**
     \brief Sets Doppler velocity.
     
     \param velocity Velocity in m/s. Defaults to 343.
     */
    virtual void SetDopplerVelocity( float velocity ) = 0;

    /**
     \brief Sets gain.
     
     \param gain Gain. Defaults to 1. 0 means muted.
     */
    virtual void SetGain( float gain ) = 0;

    /**
       \brief Sets muting on/off.  Defaults to unmuted.
     
       \param mute True, if the audio will be muted.
     */
    virtual void SetMute( bool mute ) = 0;
};
#endif
