#include "KeyboardInputWin.hpp"
#include "MouseInput.hpp"
#include "RendererImpl.hpp"
#define VC_EXTRALEAN
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <map>

#pragma comment (lib, "d3d11.lib")

static std::map< int, ae3d::keyboardinput::Enum > keyCodes;
extern bool gMouseButtonDown[ 3 ];
extern int gMousePosition[ 2 ];

static void InitKeycodes()
{
    keyCodes[ VK_ESCAPE ] = ae3d::keyboardinput::KEY_ESC;
    keyCodes[ 'A' ] = ae3d::keyboardinput::KEY_a;
    keyCodes[ 'B' ] = ae3d::keyboardinput::KEY_b;
    keyCodes[ 'C' ] = ae3d::keyboardinput::KEY_c;
    keyCodes[ 'D' ] = ae3d::keyboardinput::KEY_d;
    keyCodes[ 'E' ] = ae3d::keyboardinput::KEY_e;
    keyCodes[ 'F' ] = ae3d::keyboardinput::KEY_f;
    keyCodes[ 'G' ] = ae3d::keyboardinput::KEY_g;
    keyCodes[ 'H' ] = ae3d::keyboardinput::KEY_h;
    keyCodes[ 'I' ] = ae3d::keyboardinput::KEY_i;
    keyCodes[ 'J' ] = ae3d::keyboardinput::KEY_j;
    keyCodes[ 'K' ] = ae3d::keyboardinput::KEY_k;
    keyCodes[ 'L' ] = ae3d::keyboardinput::KEY_l;
    keyCodes[ 'M' ] = ae3d::keyboardinput::KEY_m;
    keyCodes[ 'N' ] = ae3d::keyboardinput::KEY_n;
    keyCodes[ 'O' ] = ae3d::keyboardinput::KEY_o;
    keyCodes[ 'P' ] = ae3d::keyboardinput::KEY_p;
    keyCodes[ 'Q' ] = ae3d::keyboardinput::KEY_q;
    keyCodes[ 'R' ] = ae3d::keyboardinput::KEY_r;
    keyCodes[ 'S' ] = ae3d::keyboardinput::KEY_s;
    keyCodes[ 'T' ] = ae3d::keyboardinput::KEY_t;
    keyCodes[ 'U' ] = ae3d::keyboardinput::KEY_u;
    keyCodes[ 'V' ] = ae3d::keyboardinput::KEY_v;
    keyCodes[ 'W' ] = ae3d::keyboardinput::KEY_w;
    keyCodes[ 'X' ] = ae3d::keyboardinput::KEY_x;
    keyCodes[ 'Y' ] = ae3d::keyboardinput::KEY_y;
    keyCodes[ 'Z' ] = ae3d::keyboardinput::KEY_z;
    keyCodes[ 37  ] = ae3d::keyboardinput::KEY_LEFT;
    keyCodes[ 39  ] = ae3d::keyboardinput::KEY_RIGHT;
    keyCodes[ 38  ] = ae3d::keyboardinput::KEY_UP;
    keyCodes[ 40  ] = ae3d::keyboardinput::KEY_DOWN;
}

/**
 * Main message handler.
 */
LRESULT CALLBACK WindowProc( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam )
{
    switch( message )
    {   
        case WM_QUIT:
            // This message indicates a request to terminate an application and is generated when the application calls the PostQuitMessage
            break;
        case WM_DESTROY:
            // This message is sent when a window is being destroyed. It is sent to the window
            // procedure of the window being destroyed after the window is removed from the screen.
            PostQuitMessage(0);
            exit(0);
            break;
        case WM_SIZE:
            RendererImpl::Instance().Resize( LOWORD( lParam ), HIWORD( lParam ) );
            break;
        case WM_CLOSE:
            RendererImpl::Instance().Deinit();
            // This message is sent as a signal that a window or an application should terminate.
            break;
        case WM_SYSCOMMAND:
            if (wParam == SC_MINIMIZE)
            {
                // Handled by WM_SIZE.
            }
            else if (wParam == SC_RESTORE)
            {
            }
            break;
        case WM_KEYDOWN:
            if (keyCodes.find( (int)wParam ) != std::end( keyCodes ) )
            {
                KeyboardInputWin::sKeyDown[ keyCodes[ (int)wParam ] ] = true;
            }
            break;
        case WM_KEYUP:
            if (keyCodes.find( (int)wParam ) != std::end( keyCodes ) )
            {
                KeyboardInputWin::sKeyDown[ keyCodes[ (int)wParam ] ] = false;
            }
            break;
        case WM_LBUTTONDOWN:
            gMouseButtonDown[ ae3d::mouseinput::MOUSE_BUTTON_LEFT ] = true;
            break;
        case WM_RBUTTONDOWN:
            gMouseButtonDown[ ae3d::mouseinput::MOUSE_BUTTON_RIGHT ] = true;
            break;
        case WM_MBUTTONDOWN:
            gMouseButtonDown[ ae3d::mouseinput::MOUSE_BUTTON_MIDDLE ] = true;
            break;
        case WM_MOUSEWHEEL:
            break;
        case WM_MOUSEMOVE:
            gMousePosition[ 0 ] = (short)LOWORD( lParam );
            gMousePosition[ 1 ] = (short)HIWORD( lParam );

            // Check to see if the left button is held down:
            //bool leftButtonDown=wParam & MK_LBUTTON;

            // Check if right button down:
            //bool rightButtonDown=wParam & MK_RBUTTON;
            break;
        case WM_ACTIVATE:
            RendererImpl::Instance().SetFocus((wParam == WA_INACTIVE) ? false : true);
            break;
    }

    // Handles any messages the switch statement didn't.
    return DefWindowProc( hWnd, message, wParam, lParam );
}

BOOL APIENTRY DllMain( HMODULE /*hModule*/,
                       DWORD  ul_reason_for_call,
                       LPVOID /*lpReserved*/
					 )
{
    InitKeycodes();

	switch (ul_reason_for_call)
	{
	case DLL_PROCESS_ATTACH:
	case DLL_THREAD_ATTACH:
	case DLL_THREAD_DETACH:
	case DLL_PROCESS_DETACH:
		break;
	}
	return TRUE;
}

