#define GL_GLEXT_PROTOTYPES
#include <EGL/egl.h>
#include <GLES3/gl3.h>
#include <SDL2/SDL.h>
#include <SDL2/SDL_syswm.h>
#include <cassert>
#include "Log.hpp"

namespace Global
{
    int width = 800;
    int height = 600;
    EGLDisplay eglDisplay = nullptr;
    EGLSurface eglSurface = nullptr;
}

static void EGLErrorCheck( const char* info )
{
	EGLint error = eglGetError();

	if (error != EGL_SUCCESS)
	{
        Log::Print( "%s caused an EGL error: %d", info, error );
    }
}

SDL_Window* mainWindow2 = nullptr;

/**
   \param width Width in pixels or 0 if desktop width should be used.
   \param height Height in pixels or 0 if desktop height should be used.
   \param outScreeenSize If width or height is 0, desktop size is returned here in a 2-element array.
 */
bool CreateAppWindow( int width, int height, int* outScreenSize )
{
    if (outScreenSize == nullptr)
    {
        Log::Print( "CreateAppWindow: outScreenSize is nullptr." );
        return false;
    }

    if (SDL_Init( SDL_INIT_EVERYTHING ) == -1)
    {
        Log::Print( "Could not init SDL!" );
        return false;
    }

    SDL_DisplayMode displayMode;
    SDL_GetDesktopDisplayMode( 0, &displayMode );

    outScreenSize[ 0 ] = width == 0 ? displayMode.w : width;
    outScreenSize[ 1 ] = height == 0 ? displayMode.h : height;
    
    SDL_GL_SetAttribute( SDL_GL_CONTEXT_MAJOR_VERSION, 3 );
	SDL_GL_SetAttribute( SDL_GL_CONTEXT_MINOR_VERSION, 0 );
	SDL_GL_SetAttribute( SDL_GL_CONTEXT_PROFILE_MASK, SDL_GL_CONTEXT_PROFILE_ES );
    SDL_GL_SetAttribute( SDL_GL_CONTEXT_EGL, 1 );
	SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
	SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 24 );

    const unsigned createFlags = SDL_WINDOW_OPENGL | SDL_WINDOW_SHOWN | (width == 0 ? SDL_WINDOW_FULLSCREEN_DESKTOP : 0);

	mainWindow2 = SDL_CreateWindow( "Aether3D App", SDL_WINDOWPOS_UNDEFINED, SDL_WINDOWPOS_UNDEFINED,
                                   outScreenSize[ 0 ], outScreenSize[ 1 ], createFlags );
    if (!mainWindow2)
    {
        Log::Print( "Could not create SDL window!" );
        return false;
    }

	SDL_ShowWindow( mainWindow2 );

    SDL_SysWMinfo sysInfo;
    SDL_VERSION( &sysInfo.version );
    SDL_GetWindowWMInfo( mainWindow2, &sysInfo );
    Global::eglDisplay = eglGetDisplay((EGLNativeDisplayType)sysInfo.info.x11.display);

    if (Global::eglDisplay == EGL_NO_DISPLAY)
    {
        assert( !"eglDisplay is not ok." );
        return false;
    }

    EGLint nNumConfigs = 0;
    if (eglGetConfigs( Global::eglDisplay, nullptr, 0, &nNumConfigs ) == EGL_FALSE)
    {
        assert( !"ERROR: eglGetConfigs failed 1.\n" );
        return false;
    }

    EGLConfig config;
    if (eglGetConfigs( Global::eglDisplay, &config, 1, &nNumConfigs ) == EGL_FALSE)
    {
        assert( !"ERROR: eglGetConfigs failed 2.\n" );
        return false;
    }
    Global::eglSurface = eglCreateWindowSurface( Global::eglDisplay, config, (NativeWindowType)sysInfo.info.x11.window, nullptr );
    EGLErrorCheck( "eglCreateWindowAndSurface" );
    return true;
}

bool CreateContext()
{
    SDL_GLContext context = SDL_GL_CreateContext( mainWindow2 );

    if (!context)
    {
        Log::Print( "Could not create SDL OpenGL ES 3.0 context!" );
        return false;
    }

    int major, minor;
    SDL_GL_GetAttribute( SDL_GL_CONTEXT_MAJOR_VERSION, &major );
    SDL_GL_GetAttribute( SDL_GL_CONTEXT_MINOR_VERSION, &minor );
    Log::Print( "Created SDL OpenGL ES %d.%d context.", major, minor );
    SDL_GL_SetSwapInterval( 1 );
    return true;
}

void EGLContextSwapBuffers()
{
    SDL_GL_SwapWindow( mainWindow2 );
}

void EGLDeinit()
{
    //eglMakeCurrent( Global::eglDisplay, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT );
	//eglTerminate( Global::eglDisplay );
}
