#ifndef AUDIOIMPL_H
#define AUDIOIMPL_H

#ifndef AUDIO_H
#    include "Audio.hpp"
#endif

/// Handles audio.
class AudioImpl : public Audio
{
  public:
    /**
     Reloads a file if it exists.

     \param path Path.
     */
    static void Reload( const char* path );

    /// Constructor. Opens the audio device.
    AudioImpl();

    /// Destructor.
    virtual ~AudioImpl();

    /**
       Adds a sound. It can be played back using play().

       \param soundName Sound's name.
       \param soundFile Sound's file name.  Format can be .wav or .ogg.
     */
    void AddSound( const char* soundName, const char* soundFile ) override;

    /**
       Plays a sound. It must have been added before using addSound().

       \param soundName Sound name.
     */
    void Play( const char* soundName ) override;
    
    /**
       \return True, if the audio is muted, false otherwise.
     */
    bool IsMuted() const override;
  
    /**
     \brief Sets Doppler factor.
     
     \param factor Factor. Defaults to 1.
     */
    void SetDopplerFactor( float factor ) override;

    /**
     \brief Sets Doppler velocity.
     
     \param velocity Velocity in m/s. Defaults to 343.
     */
    void SetDopplerVelocity( float velocity ) override;

    /**
     \brief Sets gain.
     
     \param gain Gain. Defaults to 1. 0 means muted.
     */
    void SetGain( float gain ) override;

    /**
       Sets muting on/off.
     
       \param mute True, if the audio will be muted.
     */
    void SetMute( bool mute ) override;
};
#endif
