#ifndef CAMERA_IMPL_H
#define CAMERA_IMPL_H

#ifndef INCLUDE_MEMORY_H
#    include <memory>
#    define INCLUDE_MEMORY_H
#endif
#ifndef CAMERA_H
#    include "Camera.hpp"
#endif

class Frustum;
struct Matrix44;
struct Vec3;
class Texture;

/// Camera's implementation.
class CameraImpl : public Camera
{
  public:
    CameraImpl();
    virtual ~CameraImpl();
    void LookAt( const Vec3& eye, const Vec3& center, const Vec3& up ) override;
    void MoveForward( float amount ) override;
    void MoveRight( float amount ) override;
    void MoveUp( float amount ) override;
    void OffsetRotation( const Vec3& axis, float angleDegrees ) override;
    Vec3 ScreenPoint( const Vec3& worldPoint ) const override;
    void SetProjection( float fieldOfViewDegrees, float aspect, float nearDepth, float farDepth ) override;
    void SetProjection( float left, float right, float bottom, float top, float nearDepth, float farDepth ) override;
    void SetViewport( int left, int right, int top, int bottom ) override;
    void SetRenderTarget( Texture* target ) override;
    Vec3 ViewDirection() const override;
    float FarClipPlane() const override;
    float NearClipPlane() const override;
    float Aspect() const;
    float FOV() const;
    Texture* GetRenderTarget() const;
    void GetViewport( int& outLeft, int& outRight, int& outTop, int& outBottom ) const;
    const Frustum& GetFrustum() const;
    const Matrix44& ProjectionMatrix() const;
    const Matrix44& ViewMatrix() const;
    struct Impl;

  private:
    std::unique_ptr< Impl > m;
};

#endif
