#ifndef RENDERER_TYPES_H
#define RENDERER_TYPES_H

#ifndef MATRIX_H
#    include "Matrix.hpp"
#endif
#ifndef SHADER_H
#    include "Shader.hpp"
#endif
#ifndef TEXTURE_IMPL_H
#    include "TextureImpl.hpp"
#endif
#ifndef VEC3_H
#    include "Vec3.hpp"
#endif
#include "ComputeShader.hpp"

struct BuiltinShaders
{
    /** \param builtinAssetShaderPath path to shaders. example: "shaders/glsl150/" */
    Error::Enum Load( const std::string& builtinAssetShaderPath );
    
    Shader unlitShader;
    Shader skyboxShader;
    Shader depthNormalShader;
    Shader depthNormalSkinShader;
    Shader toneMapShader;
    Shader blurShader;
    Shader blurSSAOShader;
    Shader ssaoShader;
    Shader momentsShader;
    Shader momentsSkinShader;
#if AETHER3D_GL43 || AETHER3D_D3D11
    ComputeShader cullLightShader;
#endif
};

struct BuiltinTargets
{
    TextureImpl depthNormalTarget; // Linear depth and view-space normal
    TextureImpl hdrTarget; // Scene is rendered into this texture, ready for PostFX.
    TextureImpl bloomBlurHTarget;
    TextureImpl bloomBlurVTarget;
    TextureImpl ssaoTarget;
    TextureImpl ssaoBlurTarget;
    TextureImpl shadowBlurHTarget;
    TextureImpl shadowBlurVTarget;
    TextureImpl computeTarget;
};

struct RendererState
{
    Matrix44 projection;
    Matrix44 view;
    Matrix44 viewProjection;
    Vec3 cameraPosition;
};

#endif
