#ifndef VERTEX_BUFFER_H
#define VERTEX_BUFFER_H

#ifndef INCLUDE_MEMORY_H
#    include <memory>
#    define INCLUDE_MEMORY_H
#endif
#ifndef INCLUDE_VECTOR_H
#    include <vector>
#    define INCLUDE_VECTOR_H
#endif
#ifndef MATRIX_H
#    include "Matrix.hpp"
#endif

struct Face;
struct Vertex;
struct Vec3;
struct Matrix;

/// Influence that is ready to be used as a vertex attribute.
struct Influence
{
    Influence()
    {
        jointIndices[ 0 ] = jointIndices[ 1 ] = jointIndices[ 2 ] = jointIndices[ 3 ] = 0;
        weights[ 1 ] = weights[ 2 ] = weights[ 3 ] = 0;
        weights[ 0 ] = 1;
    }

    float jointIndices[ 4 ];
    float weights[ 4 ];
};

/// Vertex buffer is used to render a mesh.
class VertexBuffer
{
  public:
    VertexBuffer();
    VertexBuffer( const VertexBuffer& other );
    VertexBuffer& operator=( const VertexBuffer& other );
    virtual ~VertexBuffer();
    bool IsGenerated() const;
    void Bind() const;
    static void Deinit();
    void Draw() const;
    void DrawInstanced( int instanceCount ) const;
    void Generate( const std::vector< Face >& indices, const std::vector< Vertex >& vertices, const std::vector< Influence >& influences );
    void Generate( const std::vector< Face >& indices, const std::vector< Vertex >& vertices, const std::vector< Vec3 >& morphPositions1,
                   const std::vector< Vec3 >& morphPositions2 );
    void SetName( const char* name );
    
private:
    struct Impl;
    std::unique_ptr< Impl > m;
};

#endif
