#include <cassert>
#include "Aether3D.hpp"

int main()
{
    Aether3D_API dll;
    Error::Enum error = dll.LoadDLL( "../Renderers/libaether3d_gl32" );

    if (error == Error::DLL_NOT_FOUND)
    {
        assert( !"Could not find engine DLL!" );
        return (int)error;
    }
    else if (error == Error::FACTORY_NOT_FOUND)
    {
        assert( !"No entry point in DLL!" );
        return (int)error;
    }
    else if (error == Error::DLL_VERSION_MISMATCH)
    {
        assert( !"DLL version does not match header version!" );
        return (int)error;
    }
    
    Renderer* renderer = dll.MakeRenderer( "../Assets/builtin/" );
    assert( renderer );

    error = renderer->OpenWindow( 640, 480, false, 1, true );

    if (error != Error::NO_ERRORS)
    {
        assert( !"Could not create rendering context!" );
        return (int)error;
    }

    std::unique_ptr< KeyboardInput > keyboardInput( dll.MakeKeyboardInput() );

    renderer->SetClearColor( 1, 0, 0 );

    std::unique_ptr< Texture > textureFromAtlas1( dll.MakeTexture( "Assets/Textures/atlas_cegui.png", "Assets/Textures/atlas_cegui.xml", "granite" ) );
    std::unique_ptr< Texture > textureFromAtlas2( dll.MakeTexture( "Assets/Textures/atlas_cegui.png", "Assets/Textures/atlas_cegui.xml", "marble" ) );
    //std::unique_ptr< Texture > texture( dll.MakeTexture( "../Tests/assets/textures/test_dxt1.dds", false, true, Texture::Tiling::Repeat, Texture::Filter::Point ) );

    //std::unique_ptr< Audio > audio( dll.MakeAudio() );
    //audio->AddSound("clipName", "../Tests/assets/sounds/failed.wav");
    //audio->AddSound("clipName", "door.wav");
    //audio->Play( "clipName" );
    
    while (!keyboardInput->IsDown( ae3d::keyboardinput::KEY_ESC ))
    {
        renderer->ClearScreen();
        
        renderer->Draw( textureFromAtlas2.get(), 384, 0, Vec4( 1, 1, 1, 1 ) );
        renderer->Draw( textureFromAtlas1.get(), 128, 0, Vec4( 1, 1, 1, 1 ) );
        //renderer->Draw( texture.get(), 128, 256, 256, 256, Vec4( 1, 1, 1, 1 ) );

        renderer->Draw( "hello world", "aether3d_font", 1, 10, 300, Vec4( 1, 1, 1, 1 ) );
        renderer->DrawStatistics();

        renderer->FinishFrame();
        renderer->SwapBuffers();
    }

    renderer->Deinit();
}

