#include <iostream>
#include <cmath>
#include "Errors.hpp"
#include "Matrix.hpp"

bool TestTranspose()
{
    Matrix44 matrix;
    const float exceptedResult = 42;
    matrix.m[ 3 ] = exceptedResult;
    matrix.Transpose( matrix );
    
    return matrix.m[ 3 * 4 ] == exceptedResult;
}

bool TestMultiply()
{
    Matrix44 matrix1;
    const float m1data[] = { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16 };
    matrix1.InitFrom( m1data );

    Matrix44 matrix2;
    const float m2data[] = { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16 };
    matrix2.InitFrom( m2data );

    Matrix44 result;
    Matrix44::Multiply( matrix1, matrix2, result );

    Matrix44 expectedResult;
    const float exData[] = 
        { 
            90, 100, 110, 120,
            202, 228, 254, 280,
            314, 356, 398, 440,
            426, 484, 542, 600
        };
    expectedResult.InitFrom( exData );

    for (int i = 0; i < 16; ++i)
    {
        if (std::abs( result.m[ i ] - expectedResult.m[ i ] ) > 0.0001f)
        {
            return false;
        }
    }

    return true;
}

bool TestInverse()
{
    Matrix44 mat;
    mat.MakeProjection( 45, 4.0f / 3.0f, 1, 200 );

    Matrix44 result;
    Matrix44::Invert( mat, result );

    Matrix44 expectedResult;
    const float exData[] = 
    { 
        0.552285f, -0, -0, -0, 
        -0, 0.414214f, -0, -0, 
        -0, -0, -0, -0.4975f, 
        -0, -0, -1, 0.5025f
    };
    expectedResult.InitFrom( exData );

    for (int i = 0; i < 16; ++i)
    {
        if (std::abs( result.m[ i ] - expectedResult.m[ i ] ) > 0.0001f)
        {
            return false;
        }
    }

    return true;
}

int main()
{
    if (!TestTranspose())
    {
        std::cout << "Matrix transpose failed." << std::endl;
        return (int)Error::CONTRACT_VIOLATION;
    }

    if (!TestMultiply())
    {
        std::cout << "Matrix multiply failed." << std::endl;
        return (int)Error::CONTRACT_VIOLATION;
    }

    if (!TestInverse())
    {
        std::cout << "Matrix invert failed." << std::endl;
        return (int)Error::CONTRACT_VIOLATION;
    }

    return (int)Error::NO_ERRORS;
}

