#include <algorithm>
#include <iostream>
#include <string>
#include <vector>
#include "Aether3D.hpp"

int main( int paramCount, char* params[] )
{
    Aether3D_API dll;
    Error::Enum error = dll.LoadDLL( paramCount > 1 ? std::string( params[ 1 ] ) : "" );

    if (error != Error::NO_ERRORS)
    {
        return (int)error;
    }

    Renderer* renderer = dll.MakeRenderer( "../../Assets/builtin/" );

    if (!renderer)
    {
        return (int)Error::FACTORY_NOT_FOUND;
    }

    int width = 320;
    int height = 200;
    error = renderer->OpenWindow( width, height, false, 1, true );

    if (error != Error::NO_ERRORS)
    {
        return (int)error;
    }

    const std::string path = "../assets/models/textured_cube.ae3d";
    Model* model = dll.MakeModel( path, error );

    if (error != Error::NO_ERRORS)
    {
        std::cerr << "Could not load " << path << std::endl;
        return (int)error;
    }

    auto meshNames = model->GetMeshNames();

    if (std::find( std::begin( meshNames ), std::end( meshNames ), "Cube" ) == std::end( meshNames ))
    {
        std::cerr << "textured_cube.ae3d doesn't contain a mesh named Cube." << std::endl;
        return Error::GENERIC;
    }

    Material* material = dll.MakeMaterial();
    material->SetShaders( "unlit", "unlit", "", "" );

    // Material set/get
    model->SetMaterialForMesh( material, "Cube" );

    if (model->GetMeshMaterial( "Cube" ) != material)
    {
        std::cout << "Material returned was not material set." << std::endl;
        return Error::CONTRACT_VIOLATION;
    }

    // Substring
    Material* material2 = dll.MakeMaterial();
    material2->SetShaders( "unlit", "unlit", "", "" );

    model->SetMaterialForMeshSubstring( material2, "Cu" );

    if (model->GetMeshMaterial( "Cube" ) != material2)
    {
        std::cout << "Substring: Material returned was not material set." << std::endl;
        return Error::CONTRACT_VIOLATION;
    }

    dll.MakeModel( "../assets/models/emptyfile", error );

    if (error != Error::CORRUPTED_DATA)
    {
        return Error::CONTRACT_VIOLATION;
    }
}

