#include <cmath>
#include <iostream>
#include "Errors.hpp"
#include "SceneGraphNode.hpp"

int main()
{
    SceneGraphNode root;
    SceneGraphNode child;

    root.AddChild( &child );

    if (root.children.size() != 1)
    {
        std::cerr << "SceneGraphNode: Wrong number of children!" << std::endl;
        return Error::CONTRACT_VIOLATION;
    }

    if (child.Parent() != &root)
    {
        std::cerr << "SceneGraphNode: Wrong parent!" << std::endl;
        return Error::CONTRACT_VIOLATION;
    }

    if (!root.ContainsChild( &child ))
    {
        std::cerr << "SceneGraphNode: ContainsChild failed!" << std::endl;
        return Error::CONTRACT_VIOLATION;
    }

    root.SetName( "Root" );

    if (root.GetName() != "Root")
    {
        std::cerr << "SceneGraphNode: Name returned was not the name set!" << std::endl;
        return Error::CONTRACT_VIOLATION;
    }

    child.SetLayerMask( 0xDEADBEEF );

    if (child.GetLayerMask() != 0xDEADBEEF)
    {
        std::cerr << "SceneGraphNode: Layermask returned was not the layermask set!" << std::endl;
        return Error::CONTRACT_VIOLATION;
    }

    child.SetPosition( Vec3( 10, 0, 0 ) );

    if (child.GetPosition() != Vec3( 10, 0, 0 ) )
    {
        std::cerr << "SceneGraphNode: Position returned was not position set!" << std::endl;
        return Error::CONTRACT_VIOLATION;
    }

    SceneGraphNode root2;
    SceneGraphNode child2;

    root2.AddChild( &child2 );
    root2.AddChild( &child2 );

    if (root2.children.size() != 1)
    {
        std::cerr << "SceneGraphNode: Added same node twice!" << std::endl;
        return Error::CONTRACT_VIOLATION;
    }

    SceneGraphNode root3;
    SceneGraphNode child3;

    root3.AddChild( &child3 );
    child3.AddChild( &root3 );

    if (child3.children.size() != 0)
    {
        std::cerr << "SceneGraphNode: Added same node twice!" << std::endl;
        return Error::CONTRACT_VIOLATION;
    }

    const Quaternion rotation( Vec3( 1, 2, 3 ), 4 );
    root3.SetRotation( rotation );

    if (root3.GetRotation() != rotation)
    {
        std::cerr << "SceneGraphNode: Rotation failed!" << std::endl;
        return Error::CONTRACT_VIOLATION;
    }

    const float scale = 2;
    root3.SetScale( scale );

    if (root3.GetScale() != scale)
    {
        std::cerr << "SceneGraphNode: Scale failed!" << std::endl;
        return Error::CONTRACT_VIOLATION;
    }
    
    SceneGraphNode namedNode;
    namedNode.SetName( "mutsis" );
    root3.AddChild( &namedNode );
    SceneGraphNode* foundNode = nullptr;
    root3.FindFirstChildWithName( "mutsis", &foundNode );

    if (!foundNode)
    {
        std::cerr << "SceneGraphNode: FindFirstChildWithName failed!" << std::endl;
        return Error::CONTRACT_VIOLATION;
    }
}

